/**
 *  \file
 *  \author:    Joshua J Bialkowski (jbialk@mit.edu)
 *  \date:      Oct 28, 2009
 *  \brief:
 *
 *  detail:
 */

#include "CTesterFrame.h"

#include <iostream>
#include <fstream>
#include <string>
#include <sstream>

#include <wx/filedlg.h>

#include "IllegalArgumentException.h"

namespace txduino
{







CTesterFrame::CTesterFrame( wxWindow* parent ) :
    CTesterFrameBase( parent ),
    m_tx(0),        // initialize null so we know we're not connected
    m_play(false)   // don't send commands to the device until user says so
{
    m_minSpins[0] = m_minSpin0;
    m_minSpins[1] = m_minSpin1;
    m_minSpins[2] = m_minSpin2;
    m_minSpins[3] = m_minSpin3;
    m_minSpins[4] = m_minSpin4;
    m_minSpins[5] = m_minSpin5;
    m_minSpins[6] = m_minSpin6;
    m_minSpins[7] = m_minSpin7;

    m_neutSpins[0] = m_neutSpin0;
    m_neutSpins[1] = m_neutSpin1;
    m_neutSpins[2] = m_neutSpin2;
    m_neutSpins[3] = m_neutSpin3;
    m_neutSpins[4] = m_neutSpin4;
    m_neutSpins[5] = m_neutSpin5;
    m_neutSpins[6] = m_neutSpin6;
    m_neutSpins[7] = m_neutSpin7;

    m_maxSpins[0] = m_maxSpin0;
    m_maxSpins[1] = m_maxSpin1;
    m_maxSpins[2] = m_maxSpin2;
    m_maxSpins[3] = m_maxSpin3;
    m_maxSpins[4] = m_maxSpin4;
    m_maxSpins[5] = m_maxSpin5;
    m_maxSpins[6] = m_maxSpin6;
    m_maxSpins[7] = m_maxSpin7;

}







void CTesterFrame::attemptDisconnect( wxCloseEvent& event )
{
    // if we're already connected to a device, we need to disconnect
    if(m_tx)
    {
        // print a message to the console indicating the disconnect
        m_consoleTxt->AppendText(
                "Disconnecting from: " +
                m_tx->getName() + "\n" );
        delete m_tx;
        m_tx = 0;
    }

    event.Skip();
}









void CTesterFrame::attemptConnect( wxCommandEvent& event )
{
    using std::cout;
    using std::endl;

	// if we're already connected to a device, we need to disconnect
    if(m_tx)
    {
        // print a message to the console indicating the disconnect
        m_consoleTxt->AppendText(
                "Disconnecting from: " +
                m_tx->getName() + "\n" );
        delete m_tx;
        m_tx = 0;
    }

    // try to connect to the device that the user has entered in the
    // text box
    try
    {
        // also print a little message saying what we're doing
        m_consoleTxt->AppendText(
                "Connecting to device: " +
                m_devTxt->GetLabelText() + "\n" );

        m_tx = new CTxDuino( m_devTxt->GetLabelText().c_str() );

        // if we can, then print a nice message to the console
        m_consoleTxt->AppendText("   Success!\n" );

        // now we need to initialize all the channels
        for(int i=0; i < 8; i++)
        {
            m_tx->setMinSat( i, (u8)m_minSpins[i]->GetValue() );
            m_tx->setNeutral( i, (u8)m_neutSpins[i]->GetValue() );
            m_tx->setRaw( i, (u8)m_neutSpins[i]->GetValue() );
            m_tx->setMaxSat( i, (u8)m_maxSpins[i]->GetValue() );
        }

        if(m_play)
            m_tx->send();
    }

    // if we can't connect to the device, then print a message to the console
    catch(IllegalArgumentException ex)
    {
        m_consoleTxt->AppendText("   Failed :(\n" );
        m_tx = 0;
    }
}











void CTesterFrame::setValueSpin( wxSpinEvent& event )
{
    if(!m_tx)
        return;

    for(int i=0; i < 8; i++)
    {
        if( event.GetEventObject() == m_minSpins[i] )
        {
            m_tx->setMinSat( i, (u8)m_minSpins[i]->GetValue() );
            m_tx->setRaw( i, (u8)m_minSpins[i]->GetValue() );
        }

        else if( event.GetEventObject() == m_neutSpins[i] )
        {
            m_tx->setNeutral( i, (u8)m_neutSpins[i]->GetValue() );
            m_tx->setRaw( i, (u8)m_neutSpins[i]->GetValue() );
        }

        else if( event.GetEventObject() == m_maxSpins[i] )
        {
            m_tx->setMaxSat( i, (u8)m_maxSpins[i]->GetValue() );
            m_tx->setRaw( i, (u8)m_maxSpins[i]->GetValue() );
        }
    }

    if(m_play)
        m_tx->send();
}












void CTesterFrame::setValue( wxCommandEvent& event )
{
    if(!m_tx)
        return;

    for(int i=0; i < 8; i++)
    {
        if( event.GetEventObject() == m_minSpins[i] )
        {
            m_tx->setMinSat( i, (u8)m_minSpins[i]->GetValue() );
            m_tx->setRaw( i, (u8)m_minSpins[i]->GetValue() );
        }

        else if( event.GetEventObject() == m_neutSpins[i] )
        {
            m_tx->setNeutral( i, (u8)m_neutSpins[i]->GetValue() );
            m_tx->setRaw( i, (u8)m_neutSpins[i]->GetValue() );
        }

        else if( event.GetEventObject() == m_maxSpins[i] )
        {
            m_tx->setMaxSat( i, (u8)m_maxSpins[i]->GetValue() );
            m_tx->setRaw( i, (u8)m_maxSpins[i]->GetValue() );
        }
    }

    if(m_play)
        m_tx->send();
}









void CTesterFrame::pause( wxCommandEvent& event )
{
	m_play = false;

	m_consoleTxt->AppendText("The device is now paused, changes will not be sent.\n" );
}











void CTesterFrame::play( wxCommandEvent& event )
{
	m_play = true;
	if(m_tx)
	    m_tx->send();

	m_consoleTxt->AppendText("The device is now active, changes will be sent.\n" );
}












void CTesterFrame::save( wxCommandEvent& event )
{
    using std::ofstream;
    using std::endl;

    wxFileDialog* saveDialog =
	        new wxFileDialog( this,
                                _("Save configuration"),
                                wxEmptyString,
                                wxEmptyString,
                                _("Comma Separated (*.csv)|*.csv|"
                                    "Text files (*.txt)|*.txt|"
                                    "All Files (*.*)|*.*"),
                                wxFD_SAVE,
                                wxDefaultPosition );

	// this will be true if the user said "OK" and false if they hit "Cancel"
	if( saveDialog->ShowModal() == wxID_OK )
	{
	    ofstream saveFile;
	    saveFile.open(saveDialog->GetPath());

	    // print column headers
	    saveFile << "\"channel\",\"min saturation\","
                    "\"neutral\",\"max saturation\"" << endl;

	    // print each channel's information
	    for(int i=0; i < 8; i++)
	    {
	        saveFile << i << ","
                     << m_minSpins[i]->GetValue() << ","
                     << m_neutSpins[i]->GetValue() << ","
                     << m_maxSpins[i]->GetValue() << endl;
	    }
	    saveFile.close();
	}

	saveDialog->Destroy();
}









void CTesterFrame::load( wxCommandEvent& event )
{
    using std::ifstream;
    using std::string;
    using std::getline;
    using std::istringstream;
    using std::ostringstream;
    using std::vector;

    wxFileDialog* openDialog =
            new wxFileDialog( this,
                                _("Load configuration"),
                                wxEmptyString,
                                wxEmptyString,
                                _("Comma Separated (*.csv)|*.csv|"
                                    "Text files (*.txt)|*.txt|"
                                    "All Files (*.*)|*.*"),
                                wxFD_OPEN,
                                wxDefaultPosition );

    // this will be true if the user said "OK" and false if they hit "Cancel"
    if( openDialog->ShowModal() == wxID_OK )
    {
        ifstream    openFile;
        string      csvLine;
        s32         chan        = 0;

        m_consoleTxt->AppendText(
                "Reading data from file: " +
                openDialog->GetFilename() + "\n" );

        openFile.open(openDialog->GetPath().c_str());

        // ignore the first line
        getline(openFile, csvLine);
        m_consoleTxt->AppendText("   ");
        m_consoleTxt->AppendText( csvLine );
        m_consoleTxt->AppendText( "\n" );

        // read every line from the stream
        while( getline(openFile, csvLine) )
        {
            istringstream   csvStream(csvLine);
            vector<string>  csvColumn;
            string          csvElement;

            m_consoleTxt->AppendText("   ");
            m_consoleTxt->AppendText(csvLine);
            m_consoleTxt->AppendText( "\n" );

            // read every element from the line that is separated by commas
            // and put it into the vector of strings
            while( getline(csvStream, csvElement, ',') )
                csvColumn.push_back(csvElement);

            if( csvColumn.size() < 4 )
            {
                m_consoleTxt->AppendText("      !!invalid line, ignored\n");
                continue;
            }

            istringstream   elmntStream(csvColumn[0]);
            elmntStream >> chan;

            if( chan >= 0 && chan < 8 )
            {
                m_minSpins[chan]->SetValue(csvColumn[1]);
                m_neutSpins[chan]->SetValue(csvColumn[2]);
                m_maxSpins[chan]->SetValue(csvColumn[3]);
            }
        }

        openFile.close();
    }

    openDialog->Destroy();
}









void CTesterFrame::saveDefault( wxCommandEvent& event )
{
    using std::ofstream;
    using std::endl;

    // this will be true if the user said "OK" and false if they hit "Cancel"
    ofstream saveFile;
    saveFile.open("default.csv");

    // print column headers
    saveFile << "\"channel\",\"min saturation\","
                "\"neutral\",\"max saturation\"" << endl;

    // print each channel's information
    for(int i=0; i < 8; i++)
    {
        saveFile << i << ","
                 << m_minSpins[i]->GetValue() << ", "
                 << m_neutSpins[i]->GetValue() << ", "
                 << m_maxSpins[i]->GetValue() << endl;
    }

    saveFile.close();
}










}
